local f=CreateFrame("frame")
f:SetScript("OnEvent", 
function(self, event, ...) 
	if self[event] then
		return self[event](...)
	end
end)
f:RegisterEvent("ADDON_LOADED")

local scaleMod = 0.5
local function EJSmallBossButtons()
	local index, scale = 1, nil
	if WorldMapBossButtonFrame and WorldMapBossButtonFrame.ready then
		scale = WorldMapBossButtonFrame:GetScale()
	end
	if not scale then return end
	while _G["EJMapButton"..index] do	
		local _,_,_,x,y = _G["EJMapButton"..index]:GetPoint(1)
		_G["EJMapButton"..index]:SetScale(scale*EJSmallerButtonScale)
		if tonumber(EJSmallerButtonScale) ~= 1 then
			x = x * (1/EJSmallerButtonScale)
			y = y * (1/EJSmallerButtonScale)
			_G["EJMapButton"..index]:SetPoint("CENTER", WorldMapBossButtonFrame, "BOTTOMLEFT", x, y)
		end
		index = index + 1
	end
end

local function createOptions()
	local optionFrame = CreateFrame( "Frame", "EJSmallerButtonsOptions", UIParent );
	local title = optionFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetPoint("TOPLEFT", 16, -16)
	local version = GetAddOnMetadata("EJSmallerMapButtons","Version") or ""
	title:SetText("EJSmallerMapButtons v"..version)
	
	local scaleSlider = CreateFrame("Slider", "EJSmallerButtonsSlider", optionFrame, "OptionsSliderTemplate")
	scaleSlider:SetPoint("TOPLEFT", 16, -75)
	scaleSlider.text = _G["EJSmallerButtonsSliderText"]
	scaleSlider.low = _G["EJSmallerButtonsSliderLow"]
	scaleSlider.high = _G["EJSmallerButtonsSliderHigh"]
	scaleSlider.tooltipText = "Drag to set Boss Button scale."
	scaleSlider:SetWidth(120)
	scaleSlider:SetMinMaxValues(0.3,1.0)
	scaleSlider:SetValue(EJSmallerButtonScale)
	scaleSlider:SetValueStep(0.05)
	scaleSlider.text:SetText(string.format("Button Scale: %d%%", EJSmallerButtonScale*100))
	scaleSlider.low:SetText("0.3")
	scaleSlider.high:SetText("1.0")
	scaleSlider:SetScript("OnValueChanged", 
		function(self, value)
			EJSmallerButtonScale = value
			scaleSlider.text:SetText(string.format("Button Scale: %d%%", EJSmallerButtonScale*100))
			optionFrame.preview:SetScale(EJSmallerButtonScale)
			if EJSmallerButtonScale ~= 1 then
				optionFrame.preview:SetPoint("LEFT", scaleSlider, "RIGHT", 50*(1/EJSmallerButtonScale), 0)
			end
		end)
	
	local buttonPreview = CreateFrame("Button", "EJSmallerButtonsPreview", optionFrame, "EncounterMapButtonTemplate")
	buttonPreview:SetPoint("LEFT", scaleSlider, "RIGHT", 50, 0)
	buttonPreview.tooltipTitle = "EJSmallerMapButtons Preview"
	buttonPreview.tooltipText = "Move the slider to set your desired button size."
	buttonPreview.displayInfo = "player"
-- 	SetPortraitTexture(buttonPreview.bgImage, buttonPreview.displayInfo)
	buttonPreview:SetScale(EJSmallerButtonScale)
	if EJSmallerButtonScale ~= 1 then
		buttonPreview:SetPoint("LEFT", scaleSlider, "RIGHT", 50*(1/EJSmallerButtonScale), 0)
	end
	buttonPreview:SetScript("OnClick", nil)
	buttonPreview:SetScript("OnEnter", 
		function(self) 
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOM")
			GameTooltip:SetText(self.tooltipTitle,1,1,1);
			GameTooltip:AddLine(self.tooltipText, nil, nil, nil, 1);
			GameTooltip:Show();
		end)
	buttonPreview:SetScript("OnLeave", GameTooltip_Hide)
	buttonPreview:SetScript("OnShow", 
		function(self) 
			SetPortraitTexture(buttonPreview.bgImage, buttonPreview.displayInfo)
		end)
	optionFrame.preview = buttonPreview
	buttonPreview:Show()
	
	optionFrame.name = "EJSmallerMapButtons"; 
	InterfaceOptions_AddCategory(optionFrame);
end

function f.ADDON_LOADED(...)
	local addon = ...
	if addon == "EJSmallerMapButtons" then
		EJSmallerButtonScale = EJSmallerButtonScale or scaleMod
 		createOptions()
		hooksecurefunc("EncounterJournal_AddMapButtons",EJSmallBossButtons)
		f:UnregisterEvent("ADDON_LOADED")
	end
end