-- Part of ReforgeLite by d07.RiV (Iroared)
-- All rights reserved

local REFORGE_COEFF = 0.4
local REFORGE_CHEAT = 5

local function DeepCopy (t, cache)
  if type (t) ~= "table" then
    return t
  end
  local copy = {}
  for i, v in pairs (t) do
    if type (v) ~= "table" then
      copy[i] = v
    else
      cache = cache or {}
      cache[t] = copy
      if cache[v] then
        copy[i] = cache[v]
      else
        copy[i] = DeepCopy (v, cache)
      end
    end
  end
  return copy
end

---------------------------------------------------------------------------------------

function ReforgeLite:DiminishStat (rating, stat)
  return rating > 0 and 1 / (0.0152366 + 0.956 / (rating / self:RatingPerPoint (stat))) or 0
end
function ReforgeLite:GetMethodScore (method)
  local score = 0
  if method.tanking then
    score = method.stats.dodge * self.weights[self.STATS.DODGE] + method.stats.parry * self.weights[self.STATS.PARRY]
    if method.tanking == "WARRIOR" then
      score = score + method.stats.block * self.weights[self.STATS.MASTERY] + method.stats.critBlock * self.weights[self.STATS.CRITBLOCK]
    elseif method.tanking == "PALADIN" then
      score = score + method.stats.block * self.weights[self.STATS.MASTERY]
    else
      for i = 1, #self.itemStats do
        if i ~= self.STATS.DODGE and i ~= self.STATS.PARRY then
          score = score + method.stats[i] * self.weights[i]
        end
      end
    end
  else
    for i = 1, #self.itemStats do
      score = score + self:GetStatScore (i, method.stats[i])
    end
  end
  return score
end
function ReforgeLite:UpdateMethodStats (method)
  method.stats = {}
  for i = 1, #self.itemStats do
    method.stats[i] = self.itemStats[i].getter ()
  end
  local oldspi = method.stats[self.STATS.SPIRIT]
  method.stats[self.STATS.SPIRIT] = method.stats[self.STATS.SPIRIT] / self.spiritBonus
  for i = 1, #method.items do
    local item = GetInventoryItemLink ("player", self.itemData[i].slotId)
    local stats = (item and GetItemStats (item) or {})
    local reforge = (item and self:GetReforgeID (item))
    if reforge then
      local src, dst = self.reforgeTable[reforge][1], self.reforgeTable[reforge][2]
      local amount = math.floor ((stats[self.itemStats[src].name] or 0) * REFORGE_COEFF)
      method.stats[src] = method.stats[src] + amount
      method.stats[dst] = method.stats[dst] - amount
    end
    if method.items[i].src and method.items[i].dst then
      method.items[i].amount = math.floor ((stats[self.itemStats[method.items[i].src].name] or 0) * REFORGE_COEFF)
      method.stats[method.items[i].src] = method.stats[method.items[i].src] - method.items[i].amount
      method.stats[method.items[i].dst] = method.stats[method.items[i].dst] + method.items[i].amount
    end
  end
  method.stats[self.STATS.SPIRIT] = math.floor (method.stats[self.STATS.SPIRIT] * self.spiritBonus + 0.5)
  if self.s2hFactor and self.s2hFactor > 0 then
    method.stats[self.STATS.HIT] = method.stats[self.STATS.HIT] +
      math.floor ((method.stats[self.STATS.SPIRIT] - oldspi) * self.s2hFactor / 100 + 0.5)
  end
  if method.tanking then
    method.stats.dodge = GetDodgeChance () - self:DiminishStat (GetCombatRating (CR_DODGE), self.STATS.DODGE)
    method.stats.parry = GetParryChance () - self:DiminishStat (GetCombatRating (CR_PARRY), self.STATS.PARRY)
    method.stats.dodge = method.stats.dodge + self:DiminishStat (method.stats[self.STATS.DODGE], self.STATS.DODGE)
    method.stats.parry = method.stats.parry + self:DiminishStat (method.stats[self.STATS.PARRY], self.STATS.PARRY)
    if method.tanking == "WARRIOR" then
      method.stats.critBlock = (method.stats[self.STATS.MASTERY] / self:GetRatingPerPoint (self.STATS.MASTERY)) * 1.5
      method.stats.block = 20 + method.stats.critBlock
    elseif method.tanking == "PALADIN" then
      method.stats.block = 5 + (method.stats[self.STATS.MASTERY] / self:GetRatingPerPoint (self.STATS.MASTERY)) * 2.25
    end
    local unhit = 100 + 0.8 * max (0, self.pdb.targetLevel)
    if 5 + method.stats.dodge + method.stats.parry + method.stats.block > unhit then
      method.stats.block = unhit - 5 - method.stats.dodge - method.stats.parry
    end
  end
end
function ReforgeLite:FinalizeReforge (data)
  local method = data.method
  for i = 1, #method.items do
    method.items[i].reforge = nil
    if method.items[i].src and method.items[i].dst then
      local amount = math.floor (method.items[i].stats[method.items[i].src] * REFORGE_COEFF)
      for j = 1, #self.reforgeTable do
        if self.reforgeTable[j][1] == method.items[i].src and self.reforgeTable[j][2] == method.items[i].dst then
          method.items[i].reforge = j
          break
        end
      end
    end
    method.items[i].stats = nil
  end
  self:UpdateMethodStats (method)
end

function ReforgeLite:CapAllows (cap, value)
  for i = 1, #cap.points do
    if cap.points[i].method == 1 and value < cap.points[i].value then
      return false
    elseif cap.points[i].method == 2 and value > cap.points[i].value then
      return false
    end
  end
  return true
end

------------------------------------- CLASSIC REFORGE ------------------------------

function ReforgeLite:GetItemReforgeOptions (item, data)
  local opt = {}
  local best = nil
  for i = 1, #self.itemStats do
    if item.stats[i] == 0 and i ~= data.caps[1].stat and i ~= data.caps[2].stat and (best == nil or data.weights[i] > data.weights[best]) then
      best = i
    end
  end
  if best then
    local worst = nil
    local worstScore = 0
    for i = 1, #self.itemStats do
      if item.stats[i] > 0 and i ~= data.caps[1].stat and i ~= data.caps[2].stat then
        local score = (data.weights[best] - data.weights[i]) * math.floor (item.stats[i] * REFORGE_COEFF)
        if score > worstScore then
          worstScore = score
          worst = i
        end
      end
    end
    if worst then
      table.insert (opt, {src = worst, dst = best, d1 = 0, d2 = 0, score = worstScore})
    else
      table.insert (opt, {d1 = 0, d2 = 0, score = 0})
    end
  else
    table.insert (opt, {d1 = 0, d2 = 0, score = 0})
  end
  for s = 1, 2 do
    if data.caps[s].stat > 0 then
      if item.stats[data.caps[s].stat] == 0 then
        for i = 1, #self.itemStats do
          if item.stats[i] > 0 then
            local amount = math.floor (item.stats[i] * REFORGE_COEFF)
            local srcweight = (i == data.caps[3 - s].stat and 0 or data.weights[i])
            table.insert (opt, {src = i, dst = data.caps[s].stat,
              ["d" .. s] = amount, ["d" .. (3 - s)] = (i == data.caps[3 - s].stat and -amount or 0), score = -amount * srcweight})
          end
        end
      end
      if item.stats[data.caps[s].stat] > 0 and best then
        local amount = math.floor (item.stats[data.caps[s].stat] * REFORGE_COEFF)
        table.insert (opt, {src = data.caps[s].stat, dst = best,
          d1 = (s == 1 and -amount or 0), d2 = (s == 2 and -amount or 0), score = data.weights[best] * amount})
      end
    end
  end
  return opt
end
function ReforgeLite:InitReforgeClassic ()
  local method = {}
  method.items = {}
  for i = 1, #self.itemData do
    method.items[i] = {}
    method.items[i].stats = {}
    local item = GetInventoryItemLink ("player", self.itemData[i].slotId)
    local stats = (item and GetItemStats (item) or {})
    for j = 1, #self.itemStats do
      method.items[i].stats[j] = (stats[self.itemStats[j].name] or 0)
    end
  end

  REFORGE_CHEAT = self.db.reforgeCheat

  local data = {}
  data.method = method
  data.weights = DeepCopy (self.pdb.weights)
  data.caps = DeepCopy (self.pdb.caps)
  data.caps[1].init = 0
  data.caps[2].init = 0
  data.initial = {}
  for i = 1, #self.itemStats do
    data.initial[i] = self.itemStats[i].getter ()
    for j = 1, #data.method.items do
      data.initial[i] = data.initial[i] - data.method.items[j].stats[i]
    end
  end
  for i = 1, #data.method.items do
    local item = GetInventoryItemLink ("player", self.itemData[i].slotId)
    local reforge = (item and self:GetReforgeID (item))
    if reforge then
      local src, dst = self.reforgeTable[reforge][1], self.reforgeTable[reforge][2]
      local amount = math.floor (method.items[i].stats[src] * REFORGE_COEFF)
      data.initial[src] = data.initial[src] + amount
      data.initial[dst] = data.initial[dst] - amount
    end
  end
  if data.caps[1].stat > 0 then
    data.caps[1].init = data.initial[data.caps[1].stat]
    for i = 1, #data.method.items do
      data.caps[1].init = data.caps[1].init + data.method.items[i].stats[data.caps[1].stat]
    end
  end
  if data.caps[2].stat > 0 then
    data.caps[2].init = data.initial[data.caps[2].stat]
    for i = 1, #data.method.items do
      data.caps[2].init = data.caps[2].init + data.method.items[i].stats[data.caps[2].stat]
    end
  end
  if data.caps[1].stat == 0 then
    data.caps[1], data.caps[2] = data.caps[2], data.caps[1]
  end
  if data.caps[2].stat == data.caps[1].stat then
    data.caps[2].stat = 0
    data.caps[2].init = 0
  end
  if data.caps[2].stat == 0 then
    REFORGE_CHEAT = 1
  end

  if self.s2hFactor then
    data.weights[self.STATS.SPIRIT] = data.weights[self.STATS.SPIRIT] + math.floor (data.weights[self.STATS.HIT] * self.s2hFactor / 100 + 0.5)
  end
  data.weights[self.STATS.SPIRIT] = math.floor (data.weights[self.STATS.SPIRIT] * self.spiritBonus + 0.5)

  return data
end

function ReforgeLite:ComputeClassic ()
  local data = self:InitReforgeClassic ()

  local TABLE_SIZE = 10000
  local scores, codes = {}, {}
  local linit = math.floor (data.caps[1].init / REFORGE_CHEAT + math.random ()) + math.floor (data.caps[2].init / REFORGE_CHEAT + math.random ()) * TABLE_SIZE
  local reforgeOptions = {}
  scores[linit] = 0
  codes[linit] = ""
  for i = 1, #self.itemData do
    local newscores, newcodes = {}, {}
    local opt = self:GetItemReforgeOptions (data.method.items[i], data)
    reforgeOptions[i] = opt
    for k, score in pairs (scores) do
      local code = codes[k]
      local s1 = k % TABLE_SIZE
      local s2 = math.floor (k / TABLE_SIZE)
      for j = 1, #opt do
        local o = opt[j]
        local nscore = score + o.score
        local nk = s1 + math.floor (o.d1 / REFORGE_CHEAT + math.random ()) + (s2 + math.floor (o.d2 / REFORGE_CHEAT + math.random ())) * TABLE_SIZE
        if newscores[nk] == nil or nscore > newscores[nk] then
          newscores[nk] = nscore
          newcodes[nk] = code .. string.char (j)
        end
      end
    end
    scores, codes = newscores, newcodes
  end

  local bestCode = {nil, nil, nil, nil}
  local bestScore = {0, 0, 0, 0}
  for k, score in pairs (scores) do
    local s1 = data.caps[1].init
    local s2 = data.caps[2].init
    local code = codes[k]
    for i = 1, #code do
      local b = code:byte (i)
      s1 = s1 + reforgeOptions[i][b].d1
      s2 = s2 + reforgeOptions[i][b].d2
    end
    local a1, a2 = true, true
    if data.caps[1].stat > 0 then
      a1 = a1 and self:CapAllows (data.caps[1], s1)
      score = score + self:GetCapScore (data.caps[1], s1)
    end
    if data.caps[2].stat > 0 then
      a2 = a2 and self:CapAllows (data.caps[2], s2)
      score = score + self:GetCapScore (data.caps[2], s2)
    end
    local allow = a1 and (a2 and 1 or 2) or (a2 and 3 or 4)
    if bestCode[allow] == nil or score > bestScore[allow] then
      bestCode[allow] = code
      bestScore[allow] = score
    end
  end
  scores, codes = nil, nil
  collectgarbage ("collect")
  local code = bestCode[1] or bestCode[2] or bestCode[3] or bestCode[4]
  for i = 1, #data.method.items do
    local opt = reforgeOptions[i][code:byte (i)]
    data.method.items[i].src = opt.src
    data.method.items[i].dst = opt.dst
  end
  self:FinalizeReforge (data)
  return data.method
end

----------------------------------- SPIRIT-TO-HIT REFORGE ------------------------------

function ReforgeLite:GetItemReforgeOptionsS2H (item, data)
  local opt = {}
  local best = nil
  for i = 1, #self.itemStats do
    if item.stats[i] == 0 and i ~= self.STATS.HIT and i ~= self.STATS.SPIRIT and (best == nil or data.weights[i] > data.weights[best]) then
      best = i
    end
  end
  if best then
    local worst = nil
    local worstScore = 0
    for i = 1, #self.itemStats do
      if item.stats[i] > 0 and i ~= self.STATS.HIT and i ~= self.STATS.SPIRIT then
        local score = (data.weights[best] - data.weights[i]) * math.floor (item.stats[i] * REFORGE_COEFF)
        if score > worstScore then
          worstScore = score
          worst = i
        end
      end
    end
    if worst then
      table.insert (opt, {src = worst, dst = best, d1 = 0, d2 = 0, score = worstScore})
    else
      table.insert (opt, {d1 = 0, d2 = 0, score = 0})
    end
  else
    table.insert (opt, {d1 = 0, d2 = 0, score = 0})
  end
  if item.stats[self.STATS.HIT] == 0 then
    for i = 1, #self.itemStats do
      if item.stats[i] > 0 then
        local amount = math.floor (item.stats[i] * REFORGE_COEFF)
        table.insert (opt, {src = i, dst = self.STATS.HIT, d1 = amount, d2 = (i == self.STATS.SPIRIT and -amount or 0),
          score = -amount * (i == self.STATS.SPIRIT and 0 or data.weights[i])})
      end
    end
  elseif best then
    local amount = math.floor (item.stats[self.STATS.HIT] * REFORGE_COEFF)
    table.insert (opt, {src = self.STATS.HIT, dst = best, d1 = -amount, d2 = 0, score = data.weights[best] * amount})
  end
  if item.stats[self.STATS.SPIRIT] == 0 then
    for i = 1, #self.itemStats do
      if item.stats[i] > 0 then
        local amount = math.floor (item.stats[i] * REFORGE_COEFF)
        table.insert (opt, {src = i, dst = self.STATS.SPIRIT, d1 = (i == self.STATS.HIT and -amount or 0), d2 = amount,
          score = -amount * (i == self.STATS.HIT and 0 or data.weights[i])})
      end
    end
  elseif best then
    local amount = math.floor (item.stats[self.STATS.SPIRIT] * REFORGE_COEFF)
    table.insert (opt, {src = self.STATS.SPIRIT, dst = best, d1 = 0, d2 = -amount, score = data.weights[best] * amount})
  end
  return opt
end
function ReforgeLite:InitReforgeS2H ()
  local method = {}
  method.items = {}
  for i = 1, #self.itemData do
    method.items[i] = {}
    method.items[i].stats = {}
    local item = GetInventoryItemLink ("player", self.itemData[i].slotId)
    local stats = (item and GetItemStats (item) or {})
    for j = 1, #self.itemStats do
      method.items[i].stats[j] = (stats[self.itemStats[j].name] or 0)
    end
  end

  REFORGE_CHEAT = self.db.reforgeCheat

  local data = {}
  data.method = method
  data.weights = DeepCopy (self.pdb.weights)
  data.cap = {stat = self.STATS.HIT, points = DeepCopy (self.pdb.caps[1].points), init = 0}
  data.initial = {}
  for i = 1, #self.itemStats do
    data.initial[i] = self.itemStats[i].getter ()
    if i == self.STATS.SPIRIT then
      data.initial[i] = data.initial[i] / self.spiritBonus
    end
    for j = 1, #data.method.items do
      data.initial[i] = data.initial[i] - data.method.items[j].stats[i]
    end
  end
  for i = 1, #data.method.items do
    local item = GetInventoryItemLink ("player", self.itemData[i].slotId)
    local reforge = (item and self:GetReforgeID (item))
    if reforge then
      local src, dst = self.reforgeTable[reforge][1], self.reforgeTable[reforge][2]
      local amount = math.floor (method.items[i].stats[src] * REFORGE_COEFF)
      data.initial[src] = data.initial[src] + amount
      data.initial[dst] = data.initial[dst] - amount
    end
  end
  data.initial[self.STATS.HIT] = data.initial[self.STATS.HIT] - math.floor (self.itemStats[self.STATS.SPIRIT].getter () * self.s2hFactor / 100 + 0.5)
  data.cap.init = data.initial[self.STATS.HIT]
  data.spi = math.floor (data.initial[self.STATS.SPIRIT] + 0.5)
  for i = 1, #data.method.items do
    data.cap.init = data.cap.init + data.method.items[i].stats[self.STATS.HIT]
    data.spi = data.spi + data.method.items[i].stats[self.STATS.SPIRIT]
  end
  data.initial[self.STATS.SPIRIT] = math.floor (data.initial[self.STATS.SPIRIT] * self.spiritBonus + 0.5)

  return data
end

function ReforgeLite:ComputeS2H ()
  local data = self:InitReforgeS2H ()

  local TABLE_SIZE = 10000
  local scores, codes = {}, {}
  local reforgeOptions = {}
  local linit = math.floor (data.cap.init / REFORGE_CHEAT + math.random ()) + math.floor (data.spi / REFORGE_CHEAT + math.random ()) * TABLE_SIZE
  scores[linit] = 0
  codes[linit] = ""
  for i = 1, #self.itemData do
    local newscores, newcodes = {}, {}
    local opt = self:GetItemReforgeOptionsS2H (data.method.items[i], data)
    reforgeOptions[i] = opt
    for k, score in pairs (scores) do
      local code = codes[k]
      local hit = k % TABLE_SIZE
      local spi = math.floor (k / TABLE_SIZE)
      for j = 1, #opt do
        local o = opt[j]
        local nscore = score + o.score
        local nk = hit + math.floor (o.d1 / REFORGE_CHEAT + math.random ()) + (spi + math.floor (o.d2 / REFORGE_CHEAT + math.random ())) * TABLE_SIZE
        if newscores[nk] == nil or nscore > newscores[nk] then
          newscores[nk] = nscore
          newcodes[nk] = code .. string.char (j)
        end
      end
    end
    scores, codes = newscores, newcodes
  end

  local bestCode = {nil, nil}
  local bestScore = {0, 0}
  for k, score in pairs (scores) do
    local code = codes[k]
    local hit = data.cap.init
    local spi = data.spi
    for i = 1, #code do
      local b = code:byte (i)
      hit = hit + reforgeOptions[i][b].d1
      spi = spi + reforgeOptions[i][b].d2
    end
    spi = math.floor (spi * self.spiritBonus + 0.5)
    hit = hit + math.floor (spi * self.s2hFactor / 100 + 0.5)
    local allow = self:CapAllows (data.cap, hit) and 1 or 2
    score = score + self:GetCapScore (data.cap, hit) + data.weights[self.STATS.SPIRIT] * spi
    if bestCode[allow] == nil or score > bestScore[allow] then
      bestCode[allow] = code
      bestScore[allow] = score
    end
  end
  scores, codes = nil, nil
  collectgarbage ("collect")
  local code = bestCode[1] or bestCode[2]
  for i = 1, #data.method.items do
    local opt = reforgeOptions[i][code:byte (i)]
    data.method.items[i].src = opt.src
    data.method.items[i].dst = opt.dst
  end
  self:FinalizeReforge (data)
  return data.method
end

----------------------------------- TANKING REFORGE ------------------------------

function ReforgeLite:GetItemReforgeOptionsTank (item, data)
  local opt = {}
  local best = nil
  for i = 1, #self.itemStats do
    if item.stats[i] == 0 and i ~= self.STATS.DODGE and i ~= self.STATS.PARRY and (best == nil or data.weights[i] > data.weights[best]) then
      best = i
    end
  end
  if best then
    local worst = nil
    local worstScore = 0
    for i = 1, #self.itemStats do
      if item.stats[i] > 0 and i ~= self.STATS.DODGE and i ~= self.STATS.PARRY then
        local score = (data.weights[best] - data.weights[i]) * math.floor (item.stats[i] * REFORGE_COEFF)
        if score > worstScore then
          worstScore = score
          worst = i
        end
      end
    end
    if worst then
      table.insert (opt, {src = worst, dst = best, dd = 0, dp = 0, score = worstScore})
    else
      table.insert (opt, {dd = 0, dp = 0, score = 0})
    end
  else
    table.insert (opt, {dd = 0, dp = 0, score = 0})
  end
  if item.stats[self.STATS.DODGE] == 0 then
    for i = 1, #self.itemStats do
      if item.stats[i] > 0 then
        local amount = math.floor (item.stats[i] * REFORGE_COEFF)
        table.insert (opt, {src = i, dst = self.STATS.DODGE, dd = amount, dp = (i == self.STATS.PARRY and -amount or 0),
          score = -amount * (i == self.STATS.PARRY and 0 or data.weights[i])})
      end
    end
  elseif best then
    local amount = math.floor (item.stats[self.STATS.DODGE] * REFORGE_COEFF)
    table.insert (opt, {src = self.STATS.DODGE, dst = best, dd = -amount, dp = 0, score = data.weights[best] * amount})
  end
  if data.model ~= "DRUID" then
    if item.stats[self.STATS.PARRY] == 0 then
      for i = 1, #self.itemStats do
        if item.stats[i] > 0 then
          local amount = math.floor (item.stats[i] * REFORGE_COEFF)
          table.insert (opt, {src = i, dst = self.STATS.PARRY, dd = (i == self.STATS.DODGE and -amount or 0), dp = amount,
            score = -amount * (i == self.STATS.DODGE and 0 or data.weights[i])})
        end
      end
    elseif best then
      local amount = math.floor (item.stats[self.STATS.PARRY] * REFORGE_COEFF)
      table.insert (opt, {src = self.STATS.PARRY, dst = best, dd = 0, dp = -amount, score = data.weights[best] * amount})
    end
  end
  return opt
end
function ReforgeLite:InitReforgeTank ()
  local method = {}
  method.items = {}
  for i = 1, #self.itemData do
    method.items[i] = {}
    method.items[i].stats = {}
    local item = GetInventoryItemLink ("player", self.itemData[i].slotId)
    local stats = (item and GetItemStats (item) or {})
    for j = 1, #self.itemStats do
      method.items[i].stats[j] = (stats[self.itemStats[j].name] or 0)
    end
  end
  method.tanking = self.pdb.model

  REFORGE_CHEAT = self.db.reforgeCheat

  local data = {}
  data.method = method
  if method.tanking == "WARRIOR" or method.tanking == "PALADIN" then
    data.weights = {}
    for i = 1, #self.itemStats do
      data.weights[i] = 0
    end
    data.weights[self.STATS.MASTERY] = 1
  else
    data.weights = DeepCopy (self.pdb.weights)
  end
  data.initial = {}
  for i = 1, #self.itemStats do
    data.initial[i] = self.itemStats[i].getter ()
    for j = 1, #data.method.items do
      data.initial[i] = data.initial[i] - data.method.items[j].stats[i]
    end
  end
  for i = 1, #data.method.items do
    local item = GetInventoryItemLink ("player", self.itemData[i].slotId)
    local reforge = (item and self:GetReforgeID (item))
    if reforge then
      local src, dst = self.reforgeTable[reforge][1], self.reforgeTable[reforge][2]
      local amount = math.floor (method.items[i].stats[src] * REFORGE_COEFF)
      data.initial[src] = data.initial[src] + amount
      data.initial[dst] = data.initial[dst] - amount
    end
  end
  data.init = {}
  data.init.dodge = data.initial[self.STATS.DODGE]
  if method.tanking ~= "DRUID" then
    data.init.parry = data.initial[self.STATS.PARRY]
  else
    data.init.parry = 0
    REFORGE_CHEAT = 1
  end
  data.init.mastery = data.initial[self.STATS.MASTERY]
  for i = 1, #data.method.items do
    data.init.dodge = data.init.dodge + data.method.items[i].stats[self.STATS.DODGE]
    data.init.mastery = data.init.mastery + data.method.items[i].stats[self.STATS.MASTERY]
    if method.tanking ~= "DRUID" then
      data.init.parry = data.init.parry + data.method.items[i].stats[self.STATS.PARRY]
    end
  end

  local dodgeRating = GetCombatRating (CR_DODGE)
  data.baseDodge = GetDodgeChance () - self:DiminishStat (dodgeRating, self.STATS.DODGE)
  local parryRating = GetCombatRating (CR_PARRY)
  data.baseParry = GetParryChance () - self:DiminishStat (parryRating, self.STATS.PARRY)
  data.unhit = 100 + 0.8 * max (0, self.pdb.targetLevel)

  return data
end

function ReforgeLite:ComputeTank ()
  local data = self:InitReforgeTank ()

  local TABLE_SIZE = 10000
  local scores, codes = {}, {}
  local reforgeOptions = {}
  local linit = math.floor (data.init.dodge / REFORGE_CHEAT + math.random ()) + math.floor (data.init.parry / REFORGE_CHEAT + math.random ()) * TABLE_SIZE
  scores[linit] = 0
  codes[linit] = ""
  for i = 1, #self.itemData do
    local newscores, newcodes = {}, {}
    local opt = self:GetItemReforgeOptionsTank (data.method.items[i], data)
    reforgeOptions[i] = opt
    for k, score in pairs (scores) do
      local code = codes[k]
      local dodge = k % TABLE_SIZE
      local parry = math.floor (k / TABLE_SIZE)
      for j = 1, #opt do
        local o = opt[j]
        local nscore = score + o.score
        local nk = dodge + math.floor (o.dd / REFORGE_CHEAT + math.random ()) + (parry + math.floor (o.dp / REFORGE_CHEAT + math.random ())) * TABLE_SIZE
        if newscores[nk] == nil or nscore > newscores[nk] then
          newscores[nk] = nscore
          newcodes[nk] = code .. string.char (j)
        end
      end
    end
    scores, codes = newscores, newcodes
  end

  local bestCode = {nil, nil}
  local bestScore = {0, 0}
  for k, score in pairs (scores) do
    local code = codes[k]
    local dodge_rating = data.init.dodge
    local parry_rating = data.init.parry
    for i = 1, #code do
      local b = code:byte (i)
      dodge_rating = dodge_rating + reforgeOptions[i][b].dd
      parry_rating = parry_rating + reforgeOptions[i][b].dp
    end
    local dodge = data.baseDodge + self:DiminishStat (dodgeRating, self.STATS.DODGE)
    local parry = data.baseParry + self:DiminishStat (parryRating, self.STATS.PARRY)
    if method.tanking == "WARRIOR" then
      local mastery = data.init.mastery + score
      local critBlock = (mastery / self:RatingPerPoint (self.STATS.MASTERY)) * 1.5
      local block = 20 + critBlock
      if 5 + dodge + parry + block > data.unhit then
        block = data.unhit - 5 - dodge - parry
      end
      score = dodge * data.weights[self.STATS.DODGE] + parry * data.weights[self.STATS.PARRY]
      score = score + block * data.weights[self.STATS.MASTERY] + critBlock * data.weights[self.STATS.CRITBLOCK]
    elseif method.tanking == "PALADIN" then
      local mastery = data.init.mastery + score
      local block = 5 + (mastery / self:RatingPerPoint (self.STATS.MASTERY)) * 2.25
      if 5 + dodge + parry + block > data.unhit then
        block = data.unhit - 5 - dodge - parry
      end
      score = dodge * data.weights[self.STATS.DODGE] + parry * data.weights[self.STATS.PARRY] + block * data.weights[self.STATS.MASTERY]
    elseif method.tanking == "DEATHKNIGHT" then
      score = score + dodge * data.weights[self.STATS.DODGE] + parry * data.weights[self.STATS.PARRY]
    end
    if bestCode[allow] == nil or score > bestScore[allow] then
      bestCode[allow] = code
      bestScore[allow] = score
    end
  end
  scores, codes = nil, nil
  collectgarbage ("collect")
  local code = bestCode[1] or bestCode[2]
  for i = 1, #data.method.items do
    local opt = reforgeOptions[i][code:byte (i)]
    data.method.items[i].src = opt.src
    data.method.items[i].dst = opt.dst
  end
  self:FinalizeReforge (data)
  return data.method
end

-----------------------------------------------------------------------------

function ReforgeLite:Compute ()
  if self.s2hFactor and self.s2hFactor > 0 and self.pdb.caps[1].stat == self.STATS.HIT then
    self.spiritBonus = self.spiritBonus or 1
    return self:ComputeS2H ()
  else
    return self:ComputeClassic ()
  end
end
